/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.constructmodel.adapter.phone;

import cc.niushuai.demo.designpattern.constructmodel.adapter.chargeinterface.IChargeInterface;
import cn.hutool.core.util.RandomUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * phone interface
 *
 * @author niushuai233
 * @date 2024/1/5 16:19
 * @since 0.0.1
 */
@Slf4j
public class Phone {

    private boolean in;

    private BrandEnum brand;

    private IChargeInterface chargeInterface;

    public Phone(BrandEnum brand, IChargeInterface chargeInterface) {
        this.brand = brand;
        this.chargeInterface = chargeInterface;
    }

    public void pushIn() {
        log.info("{}手机插入充电器 准备充电", brand.getName());
        if (!chargeInterface.supports(brand.getInterface())) {
            log.warn("接口不匹配, 请检查 需要: {}", brand.getInterface().name());
            return;
        }
        chargeInterface.pushIn();
        in = true;
    }

    public void pullOut() {
        if (!in) {
            return;
        }
        chargeInterface.pullOut();
        log.info("{}手机拔出充电器 当前电量: {}%", brand.getName(), RandomUtil.randomInt(50, 100));
    }
}
