/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.constructmodel.bridge.pay;

import cc.niushuai.demo.designpattern.constructmodel.bridge.security.Security;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;

/**
 * WeChat pay
 *
 * @author niushuai233
 * @date 2024/1/5 17:12
 * @since 0.0.1
 */
@Slf4j
public class WxPay extends PayWay {

    public WxPay(Security security) {
        super(security);
    }

    @Override
    public Integer transfer(String fromUid, String toUid, BigDecimal amount) {
        log.info("微信转账: 从账户{}转账到{}, 金额: {}", fromUid, toUid, amount);
        if (security.verify(fromUid)) {
            log.info("微信转账: 账户[{}]安全校验通过, 转账金额: {}, 转账成功", fromUid, amount);
            return 0;
        }
        log.warn("微信转账: 账户[{}]安全校验失败", fromUid);
        return -1;
    }
}
