/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.builder._01;

import cn.hutool.core.util.StrUtil;
import lombok.Data;

/**
 * people creator
 *
 * @author niushuai233
 * @date 2023/12/20 16:19
 * @since 0.0.1
 */
@Data
public class People {

    private String id;
    private String name;
    private String idCard;
    private Sex sex;
    private Address address;

    @Override
    public String toString() {
        return StrUtil.format("id: {}, 姓名: {}, 身份证号: {} 性别: {}, 地址: {}", id, name, idCard, sex, address);
    }

    public static class Builder {
        private People people;

        public Builder() {
            this.people = new People();
        }

        public Builder id(String id) {
            people.id = id;
            return this;
        }

        public Builder name(String name) {
            people.name = name;
            return this;
        }

        public Builder idCard(String idCard) {
            people.idCard = idCard;
            return this;
        }

        public Builder sex(Sex sex) {
            people.sex = sex;
            return this;
        }

        public Builder address(Address address) {
            people.address = address;
            return this;
        }

        public People build() {
            return people;
        }
    }
}
