/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.facade._01;

import cc.niushuai.demo.designpattern.creatormodel.facade._01.entity.AopUser;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * aop实现外观模式
 *
 * @author niushuai233
 * @date 2024/1/29 11:39
 * @since 0.0.1
 */
public interface AopUserService {

    Map<String, AopUser> users = new HashMap<String, AopUser>();

    default Set<String> userKeys() {
        return users.keySet();
    }

    default AopUser queryUserById(Integer id) {
        if (null == id) {
            return null;
        }
        return users.values().stream().filter(item -> id.equals(item.getId())).findFirst().orElse(null);
    }

    default AopUser queryUserByName(String username) {
        if (StrUtil.isEmpty(username)) {
            return null;
        }
        return users.getOrDefault(username, null);
    }

    default AopUser queryUser(Object param) {
        if (NumberUtil.isNumber(param.toString())) {
            AopUser aopUser = queryUserById((Integer) param);
            if (null != aopUser) {
                return aopUser;
            }
        }

        return queryUserByName((String) param);
    }

    default AopUser aopQueryUser(Object param) {

        return null;
    }
}
