/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.prototype._01;

import cc.niushuai.demo.designpattern.creatormodel.prototype._01.entity.*;
import cn.hutool.core.util.SerializeUtil;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 问题
 *
 * @author niushuai233
 * @date 2023/12/22 9:36
 * @since 0.0.1
 */
public class QuestionService {

    private static List<Question> questions = new ArrayList<Question>();

    static {
        initSelectQuestions();
        initJudgeQuestions();
        initAnswerQuestions();
    }

    private static void initSelectQuestions() {

        questions.add(createSelectQuestion("下列说法正确的是",
                "JAVA程序中可以有多个main⽅法",
                "JAVA程序的main⽅法必须写在类⾥⾯",
                "JAVA程序的main⽅法中如果只有⼀条语句，可以不⽤{}(⼤括号)括起来",
                "JAVA程序中类名必须与⽂件名⼀样", SelectAnswerEnum.B));
        questions.add(createSelectQuestion("表达式(11+3*8)/4%3的值是", "2", "1", "31", "0", SelectAnswerEnum.A));
        questions.add(createSelectQuestion("以下()不是合法的标识符", "void", "de$f", "x3x", "STRING", SelectAnswerEnum.A));
        questions.add(createSelectQuestion("1+1=?", "4", "2", "3", "1", SelectAnswerEnum.B));
        questions.add(createSelectQuestion("变量命名规范说法正确的是",
                "变量不能以数字作为开头",
                "A和a在java中是同⼀个变量",
                "不同类型的变量，可以起相同的名字",
                "变量由字⺟、下划线、数字、$符号随意组成", SelectAnswerEnum.D));
    }

    private static Question createSelectQuestion(String title, String optionA, String optionB, String optionC, String optionD, SelectAnswerEnum answer) {
        Question question = new Question();
        question.setType(QuestionTypeEnum.SELECT);
        question.setTitle(title);
        ArrayList<Option> options = new ArrayList<>();
        options.add(new Option(SelectAnswerEnum.A, optionA));
        options.add(new Option(SelectAnswerEnum.B, optionB));
        options.add(new Option(SelectAnswerEnum.C, optionC));
        options.add(new Option(SelectAnswerEnum.D, optionD));
        question.setAnswer(new Answer(answer));
        question.setOptions(options);

        return question;
    }

    private static void initJudgeQuestions() {
        questions.add(createJudgeQuestion("5比4多25%, 4比5少20%", true));
        questions.add(createJudgeQuestion("1+1>2", false));
        questions.add(createJudgeQuestion("1斤铁比1斤棉花重", false));
        questions.add(createJudgeQuestion("明天一定会下雨", false));
        questions.add(createJudgeQuestion("一天有24小时", true));
    }

    private static Question createJudgeQuestion(String title, Boolean answer) {
        Question question = new Question();
        question.setType(QuestionTypeEnum.JUDGE);
        question.setTitle(title);
        question.setAnswer(new Answer(answer));
        return question;
    }

    private static void initAnswerQuestions() {
        questions.add(createAnswerQuestion("接口一定要有返回值", "不一定"));
        questions.add(createAnswerQuestion("什么床不能睡觉", "⽛床"));
        questions.add(createAnswerQuestion("铁棒打头疼还是⽊棒打头疼", "头最疼"));
        questions.add(createAnswerQuestion("为什么好⻢不吃回头草", "后⾯的草没了"));
        questions.add(createAnswerQuestion("⼩红⻢和⼩⿊⻢⽣的⼩⻢⼏条腿", "4条腿"));
    }

    private static Question createAnswerQuestion(String title, String answer) {
        Question question = new Question();
        question.setType(QuestionTypeEnum.ANSWER);
        question.setTitle(title);
        question.setAnswer(new Answer(answer));
        return question;
    }

    public static List<Question> getQuestions() {
        return disturbQuestionOrder(cloneQuestions());
    }

    private static List<Question> cloneQuestions() {
        List<Question> list = new ArrayList<>(questions.size());

        for (Question question : questions) {
            Question cloneQuestion = SerializeUtil.clone(question);
            list.add(cloneQuestion);
        }

        return list;
    }

    private static List<Question> disturbQuestionOrder(List<Question> questions) {
        List<Question> list = new ArrayList<>(questions.size());

        List<Question> selectQuestions = questions.stream().filter(item -> QuestionTypeEnum.SELECT.equals(item.getType())).collect(Collectors.toList());
        // 打乱答案顺序
        selectQuestions.forEach(QuestionService::disturbAnswerOrder);
        // 选择题乱序
        Collections.shuffle(selectQuestions);

        // 判断题乱序
        List<Question> judgeQuestions = questions.stream().filter(item -> QuestionTypeEnum.JUDGE.equals(item.getType())).collect(Collectors.toList());
        Collections.shuffle(judgeQuestions);

        // 问答题乱序
        List<Question> answerQuestions = questions.stream().filter(item -> QuestionTypeEnum.ANSWER.equals(item.getType())).collect(Collectors.toList());
        Collections.shuffle(answerQuestions);

        list.addAll(selectQuestions);
        list.addAll(judgeQuestions);
        list.addAll(answerQuestions);

        return list;
    }

    private static void disturbAnswerOrder(Question question) {
        if (!QuestionTypeEnum.SELECT.equals(question.getType())) {
            return;
        }
        // 只有选择题需要乱序
        List<Option> options = question.getOptions();

        // 记录乱序前的正确答案
        String answerValue = null;
        for (int i = 0; i < options.size(); i++) {
            Option option = options.get(i);
            if (option.getKey().equals(question.getAnswer().getKey())) {
                answerValue = option.getValue();
                break;
            }
        }

        // 打乱顺序
        Collections.shuffle(options);

        // 找出乱序后的正确答案
        SelectAnswerEnum[] values = SelectAnswerEnum.values();
        for (int i = 0; i < options.size(); i++) {
            Option option = options.get(i);
            // 设置新key
            option.setKey(values[i]);
            // 如果新key是答案 记录新key
            if (option.getValue().equals(answerValue)) {
                question.setAnswer(new Answer(option.getKey()));
            }
        }
    }
}
