/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.prototype._01;

import cc.niushuai.demo.designpattern.creatormodel.prototype._01.entity.Option;
import cc.niushuai.demo.designpattern.creatormodel.prototype._01.entity.Question;
import cc.niushuai.demo.designpattern.creatormodel.prototype._01.entity.QuestionTypeEnum;
import lombok.Data;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 学生
 *
 * @author niushuai233
 * @date 2023/12/28 16:40
 * @since 0.0.1
 */
@Data
public class Student {

    private String stuNo;
    private String name;

    private List<Question> questions;

    public Student(String stuNo, String name, List<Question> questions) {
        this.stuNo = stuNo;
        this.name = name;
        this.questions = questions;
    }

    @Override
    public String toString() {

        StringBuilder sb = new StringBuilder();
        sb.append("======================================================================================").append(System.lineSeparator());
        sb.append("学号: ").append(stuNo).append(System.lineSeparator());
        sb.append("姓名: ").append(name).append(System.lineSeparator());
        sb.append("---------------------------------------------------------------------------------------").append(System.lineSeparator());
        sb.append("一、选择题").append(System.lineSeparator());

        List<Question> selectQuestions = questions.stream().filter(item -> QuestionTypeEnum.SELECT.equals(item.getType())).collect(Collectors.toList());
        for (int i = 0; i < selectQuestions.size(); i++) {
            Question question = selectQuestions.get(i);

            List<Option> options = question.getOptions();

            sb.append((i + 1) + "、\t").append(question.getTitle()).append(System.lineSeparator());
            for (Option option : options) {
                sb.append("\t").append(option.getKey()).append(": ").append(option.getValue()).append(System.lineSeparator());
            }
            sb.append("答案: ").append(question.getAnswer().getKey()).append(System.lineSeparator());
        }
        sb.append(System.lineSeparator());

        sb.append("二、判断题").append(System.lineSeparator());
        List<Question> judgeQuestions = questions.stream().filter(item -> QuestionTypeEnum.JUDGE.equals(item.getType())).collect(Collectors.toList());
        for (int i = 0; i < judgeQuestions.size(); i++) {
            Question question = judgeQuestions.get(i);
            sb.append((i + 1) + "、\t").append(question.getTitle()).append("  ( )").append(System.lineSeparator());
            sb.append("答案: ").append(question.getAnswer().getCorrect()).append(System.lineSeparator());
        }
        sb.append(System.lineSeparator());

        sb.append("二、判断题").append(System.lineSeparator());
        List<Question> answerQuestions = questions.stream().filter(item -> QuestionTypeEnum.ANSWER.equals(item.getType())).collect(Collectors.toList());
        for (int i = 0; i < answerQuestions.size(); i++) {
            Question question = answerQuestions.get(i);
            sb.append((i + 1) + "、\t").append(question.getTitle()).append("  ( )").append(System.lineSeparator());
            sb.append("答案: ").append(question.getAnswer().getAnswer()).append(System.lineSeparator());
        }
        sb.append("======================================================================================").append(System.lineSeparator());
        sb.append(System.lineSeparator());


        return sb.toString();
    }
}
