/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.singleton;

import cn.hutool.core.thread.ThreadUtil;

import java.util.concurrent.atomic.AtomicReference;

/**
 * TODO
 *
 * @author niushuai233
 * @date 2024/1/5 11:49
 * @since 0.0.1
 */
public class CASSingleton {

    private static final AtomicReference<CASSingleton> INSTANCE = new AtomicReference<>();

    private CASSingleton() {
        ThreadUtil.sleep(3000);
        System.out.println(Thread.currentThread().getName() + " -> " + getClass().getName() + " has been initialized");
    }

    public static CASSingleton getInstance() {

        CASSingleton casSingleton = INSTANCE.get();
        if (casSingleton != null) {
            return casSingleton;
        }
        INSTANCE.compareAndSet(null, new CASSingleton());

        return INSTANCE.get();
    }

    public static CASSingleton getInstance_forEach() {

        while (true) {
            CASSingleton casSingleton = INSTANCE.get();
            if (casSingleton != null) {
                return casSingleton;
            }
            if (INSTANCE.compareAndSet(null, new CASSingleton())) {
                return INSTANCE.get();
            }
        }
    }

    public void hello(String world) {
        System.out.println("hello " + world);
    }
}
