/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.singleton.lazy;

import cn.hutool.core.date.DateUtil;

/**
 * 懒汉模式
 *
 * @author niushuai233
 * @date 2024/1/5 11:14
 * @since 0.0.1
 */
public class LazySingleton {

    private static LazySingleton INSTANCE;

    private LazySingleton() {
        System.out.println(Thread.currentThread().getName() + " -> " + getClass().getName() + " has been initialized");
    }

    /**
     * 获取LazySingleton 线程不安全 存在同时获取多个实例时new了多个instance到内存中
     *
     * @return
     * @author niushuai233
     * @date 2024/1/5 11:15
     * @since 0.0.1
     */
    public static LazySingleton getInstance_threadUnsafe() {
        if (null == INSTANCE) {
            INSTANCE = new LazySingleton();
        }
        return INSTANCE;
    }


    /**
     * 获取LazySingleton 线程安全
     *
     * @return
     * @author niushuai233
     * @date 2024/1/5 11:16
     * @since 0.0.1
     */
    public static synchronized LazySingleton getInstance_threadSafe() {
        if (null == INSTANCE) {
            INSTANCE = new LazySingleton();
        }
        return INSTANCE;
    }

    public void hello(String str) {
        System.out.println(Thread.currentThread().getName() + ": hello " + str + " ==>" + DateUtil.now());
    }
}
