/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.singleton.lazy;

import cn.hutool.core.date.DateUtil;

/**
 * 使用内部类的方式进行懒加载 自身线程安全 不需要锁来管控线程安全
 *
 * @author niushuai233
 * @date 2024/1/5 11:21
 * @since 0.0.1
 */
public class LazySingletonInnerClass {

    public static LazySingletonInnerClass INSTANCE = new LazySingletonInnerClass();

    private LazySingletonInnerClass() {
        System.out.println(Thread.currentThread().getName() + " -> " + getClass().getName() + " has been initialized");
    }

    public static void say() {
        INSTANCE.hello("say");
    }

    /**
     * 自身线程安全 不需要锁 只有加载内部类时单例才会被实例化
     *
     * @return
     * @author niushuai233
     * @date 2024/1/5 11:23
     * @since 0.0.1
     */
    public static LazySingletonInnerClass getInstance() {
        return LazySingletonInnerClassHolder.INSTANCE;
    }

    public void hello(String str) {
        System.out.println(Thread.currentThread().getName() + ": hello " + str + " ==>" + DateUtil.now());
    }

    public static class LazySingletonInnerClassHolder {

        private LazySingletonInnerClassHolder() {
            System.out.println(Thread.currentThread().getName() + " -> " + getClass().getName() + " has been initialized");
        }

        public static final LazySingletonInnerClass INSTANCE = new LazySingletonInnerClass();
    }
}
