/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.abstractfactory._01;

import cc.niushuai.demo.designpattern.creatormodel.abstractfactory._01.mobile.IMobileProduct;
import cc.niushuai.demo.designpattern.creatormodel.abstractfactory._01.pc.IPcProduct;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;

/**
 * product factory
 *
 * @author niushuai233
 * @date 2023/12/20 15:28
 * @since 0.0.1
 */
@Slf4j
public class ProductFactoryTest {

    @Test
    public void testProduct() {

        log.info("================xiaomi product demo================");
        XiaomiFactory xiaomiFactory = new XiaomiFactory();
        testPhone(xiaomiFactory.producePhone());
        testPc(xiaomiFactory.producePc());

        log.info("================OPPO product demo================");
        OPPOFactory oppoFactory = new OPPOFactory();
        testPhone(oppoFactory.producePhone());
        testPc(oppoFactory.producePc());
    }

    private void testPc(IPcProduct pc) {
        pc.start();
        pc.surfing();
        pc.shutdown();
    }

    private void testPhone(IMobileProduct phone) {
        phone.start();
        phone.call("13800138000");
        phone.sendSMS("13800138000", "pls say hello.");
        phone.shutdown();
    }
}
