/*
 * Copyright (C) 2023 niushuai233 niushuai.cc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.niushuai.demo.designpattern.creatormodel.singleton;

import cc.niushuai.demo.designpattern.creatormodel.singleton.lazy.LazySingleton;
import cc.niushuai.demo.designpattern.creatormodel.singleton.lazy.LazySingletonInnerClass;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.concurrent.ForkJoinPool;

/**
 * 单例模式 test
 *
 * @author niushuai233
 * @date 2024/1/5 14:05
 * @since 0.0.1
 */
public class SingletonTest {

    @Test
    public void testSingleton() {
        Singleton.map.put("date", DateUtil.now());
        System.out.println(Singleton.map.get("date"));
    }


    @Test
    public void testSingleton_lazy_unsafe() {

        List<String> strings = List.of("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12");

        new ForkJoinPool().submit(() -> strings.parallelStream().forEach(item -> LazySingleton.getInstance_threadUnsafe().hello(item)));
        ThreadUtil.sleep(3000);
        System.out.println("testSingleton_lazy_unsafe end");
    }

    @Test
    public void testSingleton_lazy_safe() {

        List<String> strings = List.of("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12");

        new ForkJoinPool().submit(() -> strings.parallelStream().forEach(item -> LazySingleton.getInstance_threadSafe().hello(item)));
        ThreadUtil.sleep(3000);
        System.out.println("testSingleton_lazy_unsafe end");
    }

    @Test
    public void testSingletonInnerClass() {

        LazySingletonInnerClass.say();
        System.out.println("============================================================================================");

        List<String> strings = List.of("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12");
        new ForkJoinPool().submit(() -> strings.parallelStream().forEach(item -> LazySingletonInnerClass.getInstance().hello(item)));

        ThreadUtil.sleep(3000);
        System.out.println("testSingletonInnerClass end");
    }

    /**
     * 仍然有可能创建出多个对象，但是INSTANCE一旦被赋予 则不再被改变 其他已创建的对象可认同为无用对象 并且等待被GC回收
     *
     * @return
     * @author niushuai233
     * @date 2024/1/5 14:54
     * @since 0.0.1
     */
    @Test
    public void testCASSingleton() {

        List<String> strings = List.of("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12");
        new ForkJoinPool().submit(() -> strings.parallelStream().forEach(item -> CASSingleton.getInstance().hello(item)));

        CASSingleton.getInstance().hello("world");

        ThreadUtil.sleep(3000);
        System.out.println("testSingletonInnerClass end");
    }

    @Test
    public void testEnumSingleton() {

        List<String> strings = List.of("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12");
        new ForkJoinPool().submit(() -> strings.parallelStream().forEach(item -> EnumSingleton.INSTANCE.hello(item)));

        EnumSingleton.INSTANCE.hello("world");

        ThreadUtil.sleep(3000);
        System.out.println("testEnumSingleton end");
    }
}
